@@ -189,7 +189,7 @@ ip4_private_init (CEPageIP4 *self, NMConnection *connection)
 		                    -1);
 	}
 
-	/* At the moment, Disabled is only supported for Wired & WiFi */
+	/* At the moment, Disabled is only supported for Ethernet & Wi-Fi */
 	if (   priv->connection_type == NM_TYPE_SETTING_WIRED
 	    || priv->connection_type == NM_TYPE_SETTING_WIRELESS) {
 		gtk_list_store_append (priv->method_store, &iter);
@@ -294,9 +294,9 @@ method_changed (GtkComboBox *combo, gpointer user_data)
 
 	gtk_widget_set_sensitive (priv->dns_servers_label, dns_enabled);
 	if (method_auto)
-		gtk_label_set_text_with_mnemonic (GTK_LABEL (priv->dns_servers_label), _("Additional _DNS servers:"));
+		gtk_label_set_text_with_mnemonic (GTK_LABEL (priv->dns_servers_label), _("Additional DNS ser_vers:"));
 	else
-		gtk_label_set_text_with_mnemonic (GTK_LABEL (priv->dns_servers_label), _("_DNS servers:"));
+		gtk_label_set_text_with_mnemonic (GTK_LABEL (priv->dns_servers_label), _("DNS ser_vers:"));
 	gtk_widget_set_sensitive (GTK_WIDGET (priv->dns_servers), dns_enabled);
 	if (!dns_enabled)
 		gtk_entry_set_text (priv->dns_servers, "");
@@ -742,9 +742,26 @@ key_pressed_cb (GtkWidget *widget,
 	#define GDK_KEY_Tab GDK_Tab
 #endif
 
-	/* Tab should behave the same way as Enter (finish editing) */
-	if (event->type == GDK_KEY_PRESS && event->key.keyval == GDK_KEY_Tab)
-		gtk_cell_editable_editing_done (GTK_CELL_EDITABLE (widget));
+	GdkKeymapKey *keys = NULL;
+	gint n_keys;
+
+	/*
+	 * Tab should behave the same way as Enter (cycling on cells).
+	 *
+	 * Previously, we had finished cell editing, which appeared to work:
+	 *   gtk_cell_editable_editing_done (GTK_CELL_EDITABLE (widget));
+	 * But unfortunately, it showed up crash occurred with XIM input (GTK_IM_MODULE=xim).
+	 * https://bugzilla.redhat.com/show_bug.cgi?id=747368
+	 */
+	if (event->type == GDK_KEY_PRESS && event->key.keyval == GDK_KEY_Tab) {
+		/* Get hardware keycode for GDK_KEY_Return */
+		if (gdk_keymap_get_entries_for_keyval (gdk_keymap_get_default (), GDK_KEY_Return, &keys, &n_keys)) {
+			/* Change 'Tab' to 'Enter' key */
+			event->key.keyval = GDK_KEY_Return;
+			event->key.hardware_keycode = keys[0].keycode;
+		}
+		g_free (keys);
+	}
 
 	return FALSE;
 }
@@ -976,6 +993,7 @@ CEPage *
 ce_page_ip4_new (NMConnection *connection,
                  GtkWindow *parent_window,
                  NMClient *client,
+                 NMRemoteSettings *settings,
                  const char **out_secrets_setting_name,
                  GError **error)
 {
@@ -987,6 +1005,7 @@ ce_page_ip4_new (NMConnection *connection,
 	                                 connection,
 	                                 parent_window,
 	                                 client,
+	                                 settings,
 	                                 UIDIR "/ce-page-ip4.ui",
 	                                 "IP4Page",
 	                                 _("IPv4 Settings")));
@@ -1103,7 +1122,7 @@ ui_to_setting (CEPageIP4 *self)
 
 		/* Gateway is optional... */
 		gtk_tree_model_get (model, &tree_iter, COL_GATEWAY, &item, -1);
-		if (item && inet_pton (AF_INET, item, &tmp_gateway) <= 0) {
+		if (item && strlen (item) && inet_pton (AF_INET, item, &tmp_gateway) <= 0) {
 			g_warning ("%s: IPv4 gateway '%s' invalid!",
 			           __func__, item ? item : "<none>");
 			g_free (item);
@@ -1180,14 +1199,14 @@ ui_to_setting (CEPageIP4 *self)
 
 	/* Update setting */
 	g_object_set (priv->setting,
-				  NM_SETTING_IP4_CONFIG_METHOD, method,
-				  NM_SETTING_IP4_CONFIG_ADDRESSES, addresses,
-				  NM_SETTING_IP4_CONFIG_DNS, dns_servers,
-				  NM_SETTING_IP4_CONFIG_DNS_SEARCH, search_domains,
-				  NM_SETTING_IP4_CONFIG_IGNORE_AUTO_DNS, ignore_auto_dns,
-				  NM_SETTING_IP4_CONFIG_DHCP_CLIENT_ID, dhcp_client_id,
-				  NM_SETTING_IP4_CONFIG_MAY_FAIL, may_fail,
-				  NULL);
+	              NM_SETTING_IP4_CONFIG_METHOD, method,
+	              NM_SETTING_IP4_CONFIG_ADDRESSES, addresses,
+	              NM_SETTING_IP4_CONFIG_DNS, dns_servers,
+	              NM_SETTING_IP4_CONFIG_DNS_SEARCH, search_domains,
+	              NM_SETTING_IP4_CONFIG_IGNORE_AUTO_DNS, ignore_auto_dns,
+	              NM_SETTING_IP4_CONFIG_DHCP_CLIENT_ID, dhcp_client_id,
+	              NM_SETTING_IP4_CONFIG_MAY_FAIL, may_fail,
+	              NULL);
 	valid = TRUE;
 
 out:
